//
//  JTAppleCalendarLayout.swift
//  JTAppleCalendar
//
//  Created by JayT on 2016-03-01.
//  Copyright © 2016 OS-Tech. All rights reserved.
//

/// Methods in this class are meant to be overridden and will be called by its collection view to gather layout information.
open class JTAppleCalendarLayout: UICollectionViewLayout, JTAppleCalendarLayoutProtocol {
    let errorDelta: CGFloat = 0.0000001
    var itemSize: CGSize = CGSize.zero
    var headerReferenceSize: CGSize = CGSize.zero
    var scrollDirection: UICollectionViewScrollDirection = .horizontal
    var maxMissCount: Int = 0
    var cellCache: [Int: [UICollectionViewLayoutAttributes]] = [:]
    var headerCache: [Int: UICollectionViewLayoutAttributes] = [:]
    var sectionSize: [CGFloat] = []
    var lastWrittenCellAttribute: UICollectionViewLayoutAttributes?
    var isPreparing = false
    var stride: CGFloat = 0
    
    var maxSections: Int {
        get {
            return monthMap.count
        }
    }

    var monthMap: [Int: Int] {
        get {
            return delegate.monthMap
        }
    }
    var numberOfRows: Int {
        get {
            return delegate.numberOfRows()
        }
    }
    var strictBoundaryRulesShouldApply: Bool {
        get {
            return (thereAreHeaders || delegate.hasStrictBoundaries()) && isPreparing
        }
    }
    var thereAreHeaders: Bool {
        get {
            return !delegate.registeredHeaderViews.isEmpty
        }
    }
    
    weak var delegate: JTAppleCalendarDelegateProtocol!
    var currentHeader: (section: Int, size: CGSize)? // Tracks the current header size
    var currentCell: (section: Int, itemSize: CGSize)? // Tracks the current cell size
    var contentHeight: CGFloat = 0 // Content height of calendarView
    var contentWidth: CGFloat = 0 // Content wifth of calendarView
    var xCellOffset: CGFloat = 0
    var yCellOffset: CGFloat = 0
    var daysInSection: [Int: Int] = [:] // temporary caching
    
    init(withDelegate delegate: JTAppleCalendarDelegateProtocol) {
        super.init()
        self.delegate = delegate
    }
    
    func indexPath(direction: SegmentDestination, of section:Int, item: Int) -> IndexPath? {
        switch direction {
        case .next:
            let lastIndexPath = cellCache[section]?.last?.indexPath
            if lastIndexPath?.section == section && lastIndexPath?.item == item {
                return cellCache[section + 1]?.first?.indexPath
            } else {
                return cellCache[section]?[item + 1].indexPath
            }
        case .previous:
            if item < 1 {
                return cellCache[section - 1]?.last?.indexPath
            } else {
                return cellCache[section]?[item - 1].indexPath
            }
        default:
            return nil
        }
    }

    /// Tells the layout object to update the current layout.
    open override func prepare() {
        if !cellCache.isEmpty {
            return
        }
        isPreparing = true
        maxMissCount = scrollDirection == .horizontal ? maxNumberOfRowsPerMonth : maxNumberOfDaysInWeek
        if scrollDirection == .vertical {
            verticalStuff()
        } else {
            horizontalStuff()
        }

        // Get rid of header data if dev didnt register headers.
        // The were used for calculation but are not needed to be displayed
        if !thereAreHeaders {
            headerCache.removeAll()
        }
        daysInSection.removeAll() // Clear chache
        isPreparing = false
    }

    var testVal: CGFloat = 0
    
    func horizontalStuff() {
        
        
        var section = 0
        var totalDayCounter = 0
        var headerGuide = 0
        let fullSection = numberOfRows * maxNumberOfDaysInWeek
        var extra = 0
        
        for aMonth in delegate.monthInfo {
            for numberOfDaysInCurrentSection in aMonth.sections {
                // Generate and cache the headers
                let sectionIndexPath = IndexPath(item: 0, section: section)
                if let aHeaderAttr = layoutAttributesForSupplementaryView(ofKind: UICollectionElementKindSectionHeader, at: sectionIndexPath) {
                        headerCache[section] = aHeaderAttr
                        if strictBoundaryRulesShouldApply {
                            contentWidth += aHeaderAttr.frame.width + testVal
                            yCellOffset = aHeaderAttr.frame.height
                        }
                    }
                // Generate and cache the cells
                for item in 0..<numberOfDaysInCurrentSection {
                    let indexPath = IndexPath(item: item, section: section)
                    if let attribute = deterimeToApplyAttribs(at: indexPath) {
                        if cellCache[section] == nil {
                            cellCache[section] = []
                        }
                        cellCache[section]!.append(attribute)
                        lastWrittenCellAttribute = attribute
                        xCellOffset += attribute.frame.width

                        if strictBoundaryRulesShouldApply {
                            headerGuide += 1
                            if numberOfDaysInCurrentSection - 1 == item || headerGuide % maxNumberOfDaysInWeek == 0 {
                                // We are at the last item in the section
                                // && if we have headers
                                headerGuide = 0
                                xCellOffset = 0
                                yCellOffset += attribute.frame.height
                            }
                        } else {
                            totalDayCounter += 1
                            extra += 1
                            if totalDayCounter % fullSection == 0 { // If you have a full section
                                xCellOffset = 0
                                yCellOffset = 0
                                contentWidth += attribute.frame.width * 7
                                stride = contentWidth
                                sectionSize.append(contentWidth)
                            } else {
                                if totalDayCounter >= delegate.totalDays {
                                    contentWidth += attribute.frame.width * 7
                                    sectionSize.append(contentWidth)
                                }
                                
                                if totalDayCounter % maxNumberOfDaysInWeek == 0 {
                                    xCellOffset = 0
                                    yCellOffset += attribute.frame.height
                                }
                            }
                        }
                    }
                }
                // Save the content size for each section
                
                if strictBoundaryRulesShouldApply {
                    sectionSize.append(contentWidth)
                    stride = sectionSize[section]
                }
                section += 1
            }
        }
        contentHeight = self.collectionView!.bounds.size.height
    }

    func verticalStuff() {
        var section = 0
        var totalDayCounter = 0
        var headerGuide = 0
        for aMonth in delegate.monthInfo {
            for numberOfDaysInCurrentSection in aMonth.sections {
                // Generate and cache the headers
                let sectionIndexPath = IndexPath(item: 0, section: section)
                if strictBoundaryRulesShouldApply {
                    if let aHeaderAttr = layoutAttributesForSupplementaryView(ofKind: UICollectionElementKindSectionHeader, at: sectionIndexPath) {
                        headerCache[section] = aHeaderAttr
                        yCellOffset += aHeaderAttr.frame.height
                        contentHeight += aHeaderAttr.frame.height
                    }
                }
                // Generate and cache the cells
                for item in 0..<numberOfDaysInCurrentSection {
                    let indexPath = IndexPath(item: item, section: section)

                    if let attribute = deterimeToApplyAttribs(at: indexPath) {
                        if cellCache[section] == nil {
                            cellCache[section] = []
                        }
                        cellCache[section]!.append(attribute)
                        lastWrittenCellAttribute = attribute
                        xCellOffset += attribute.frame.width
                        if strictBoundaryRulesShouldApply {
                            headerGuide += 1
                            if headerGuide % maxNumberOfDaysInWeek == 0 || numberOfDaysInCurrentSection - 1 == item {
                                // We are at the last item in the
                                // section && if we have headers
                                headerGuide = 0
                                xCellOffset = 0
                                yCellOffset += attribute.frame.height
                                contentHeight += attribute.frame.height
                            }
                        } else {
                            totalDayCounter += 1
                            if totalDayCounter % maxNumberOfDaysInWeek == 0 {
                                xCellOffset = 0
                                yCellOffset += attribute.frame.height
                                contentHeight += attribute.frame.height
                            } else if totalDayCounter == delegate.totalDays {
                                contentHeight += attribute.frame.height
                            }
                        }
                    }
                }
                // Save the content size for each section
                sectionSize.append(contentHeight)
                section += 1
            }
        }
        contentWidth = self.collectionView!.bounds.size.width
    }

    /// Returns the width and height of the collection view’s contents.
    /// The width and height of the collection view’s contents.
    open override var collectionViewContentSize: CGSize {
        return CGSize(width: contentWidth, height: contentHeight)
    }

    /// Returns the layout attributes for all of the cells
    /// and views in the specified rectangle.
    override open func layoutAttributesForElements(in rect: CGRect) -> [UICollectionViewLayoutAttributes]? {
        let startSectionIndex = startIndexFrom(rectOrigin: rect.origin)
        // keep looping until there were no interception rects
        var attributes: [UICollectionViewLayoutAttributes] = []
        var beganIntercepting = false
        var missCount = 0
        for sectionIndex in startSectionIndex..<cellCache.count {
            if let validSection = cellCache[sectionIndex],
                !validSection.isEmpty {
                    // Add header view attributes
                    if thereAreHeaders {
                        if headerCache[sectionIndex]!.frame.intersects(rect) {
                            attributes.append(headerCache[sectionIndex]!)
                        }
                    }
                    for val in validSection {
                        if val.frame.intersects(rect) {
                            missCount = 0
                            beganIntercepting = true
                            attributes.append(val)
                        } else {
                            missCount += 1
                            // If there are at least 8 misses in a row
                            // since intercepting began, then this
                            // section has no more interceptions.
                            // So break
                            if missCount > maxMissCount &&
                                beganIntercepting {
                                break
                            }
                        }
                    }
                    if missCount > maxMissCount && beganIntercepting {
                        break
                    }// Also break from outter loop
            }
        }
        return attributes
    }

    /// Returns the layout attributes for the item at the specified index
    /// path. A layout attributes object containing the information to apply
    /// to the item’s cell.
    override open func layoutAttributesForItem(at indexPath: IndexPath) -> UICollectionViewLayoutAttributes? {
            // If this index is already cached, then return it else,
            // apply a new layout attribut to it
            if let alreadyCachedCellAttrib = cellCache[indexPath.section],
                indexPath.item < alreadyCachedCellAttrib.count,
                indexPath.item >= 0 {
                    return alreadyCachedCellAttrib[indexPath.item]
            }
            return deterimeToApplyAttribs(at: indexPath)
    }

    func deterimeToApplyAttribs(at indexPath: IndexPath) -> UICollectionViewLayoutAttributes? {
        let monthIndex = monthMap[indexPath.section]!
        let numberOfDays = numberOfDaysInSection(monthIndex)
        // return nil on invalid range
        if !(0...maxSections ~= indexPath.section) || !(0...numberOfDays  ~= indexPath.item) { // JT101 look at the ranges
            return nil
        }
        let attr = UICollectionViewLayoutAttributes(forCellWith: indexPath)
        applyLayoutAttributes(attr)
        return attr
    }

    /// Returns the layout attributes for the specified supplementary view.
    open override func layoutAttributesForSupplementaryView(ofKind elementKind: String, at indexPath: IndexPath) -> UICollectionViewLayoutAttributes? {
        let attributes = UICollectionViewLayoutAttributes(forSupplementaryViewOfKind: elementKind, with: indexPath)
        if let alreadyCachedHeaderAttrib = headerCache[indexPath.section] {
            return alreadyCachedHeaderAttrib
        }

        let headerSize = cachedHeaderSizeForSection(indexPath.section)

        switch scrollDirection {
        case .horizontal:
            let modifiedSize = sizeForitemAtIndexPath(indexPath)
            attributes.frame = CGRect(x: contentWidth, y: 0, width: modifiedSize.width * 7, height: headerSize.height)
        case .vertical:
            // Use the calculaed header size and force the width
            // of the header to take up 7 columns
            // We cache the header here so we dont call the
            // delegate so much

            let modifiedSize = CGSize(width: collectionView!.frame.width, height: headerSize.height)
            attributes.frame = CGRect(x: 0, y: yCellOffset, width: modifiedSize.width, height: modifiedSize.height)
        }
        if attributes.frame == CGRect.zero {
            return nil
        }
        return attributes
    }

    func applyLayoutAttributes(_ attributes: UICollectionViewLayoutAttributes) {
        if attributes.representedElementKind != nil {
            return
        }
        // Calculate the item size
        let size = sizeForitemAtIndexPath(attributes.indexPath)
        attributes.frame = CGRect(x: xCellOffset + stride, y: yCellOffset, width: size.width, height: size.height)
    }

    func numberOfDaysInSection(_ index: Int) -> Int {
        if let days = daysInSection[index] {
            return days
        }
        let days = delegate.monthInfo[index].numberOfDaysInMonthGrid
        daysInSection[index] = days
        return days
    }

    func cachedHeaderSizeForSection(_ section: Int) -> CGSize {
        // We cache the header here so we dont call the delegate so much
        var headerSize = CGSize.zero
        if let cachedHeader  = currentHeader, cachedHeader.section == section {
                headerSize = cachedHeader.size
        } else {
            headerSize = delegate.referenceSizeForHeaderInSection(section)
            currentHeader = (section, headerSize)
        }
        return headerSize
    }

    func sizeForitemAtIndexPath(_ indexPath: IndexPath) -> CGSize {
        if let cachedCell  = currentCell,
            cachedCell.section == indexPath.section {
            
            if !strictBoundaryRulesShouldApply, scrollDirection == .horizontal,
                !cellCache.isEmpty {
                return cellCache[0]?[0].size ?? CGSize.zero
            } else {
                return cachedCell.itemSize
            }
        }

        var size: CGSize = CGSize.zero
        if let _ = delegate.itemSize {
            if scrollDirection == .vertical {
                size = itemSize
            } else {
                size.width = itemSize.width
                var headerSize =  CGSize.zero
                if strictBoundaryRulesShouldApply {
                    headerSize =
                        cachedHeaderSizeForSection(indexPath.section)
                }

                let currentMonth = delegate.monthInfo[monthMap[indexPath.section]!]
                let recalculatedNumOfRows = delegate.allowsDateCellStretching ? CGFloat(currentMonth.maxNumberOfRowsForFull(developerSetRows: numberOfRows)) : CGFloat(maxNumberOfRowsPerMonth)
                size.height = (collectionView!.frame.height - headerSize.height) / recalculatedNumOfRows
                currentCell = (section: indexPath.section, itemSize: size)
            }
        } else {
            // Get header size if it alrady cached
            var headerSize =  CGSize.zero
            if strictBoundaryRulesShouldApply {
                headerSize = cachedHeaderSizeForSection(indexPath.section)
            }
            var height: CGFloat = 0
            let currentMonth = delegate.monthInfo[monthMap[indexPath.section]!]
            let numberOfRowsForSection: Int
            if delegate.allowsDateCellStretching {
                if strictBoundaryRulesShouldApply {
                    numberOfRowsForSection = currentMonth.maxNumberOfRowsForFull(developerSetRows: numberOfRows)
                } else {
                    numberOfRowsForSection = numberOfRows
                }
            } else {
                numberOfRowsForSection = maxNumberOfRowsPerMonth
            }
            height = (collectionView!.frame.height - headerSize.height) / CGFloat(numberOfRowsForSection)
            size        = CGSize(width: itemSize.width, height: height)
            currentCell = (section: indexPath.section, itemSize: size)
        }
        return size
    }

    func sizeOfSection(_ section: Int) -> CGFloat {
        switch scrollDirection {
        case .horizontal:
            return cellCache[section]![0].frame.width * CGFloat(maxNumberOfDaysInWeek)
        case .vertical:
            let headerSizeOfSection = !headerCache.isEmpty ? headerCache[section]!.frame.height : 0
            return cellCache[section]![0].frame.height * CGFloat(numberOfRowsForMonth(section)) + headerSizeOfSection
        }
    }

    func numberOfRowsForMonth(_ index: Int) -> Int {
        let monthIndex = monthMap[index]!
        return delegate.monthInfo[monthIndex].rows
    }

    func startIndexFrom(rectOrigin offset: CGPoint) -> Int {
        let key =  scrollDirection == .horizontal ? offset.x : offset.y
        return startIndexBinarySearch(sectionSize, offset: key)
    }

    func sizeOfContentForSection(_ section: Int) -> CGFloat {
        return sizeOfSection(section)
    }
    
//    func sectionFromRectOffset(_ offset: CGPoint) -> Int {
//        let theOffet = scrollDirection == .horizontal ? offset.x : offset.y
//        return sectionFromOffset(theOffet)
//    }

    func sectionFromOffset(_ theOffSet: CGFloat) -> Int {
        var val: Int = 0
        for (index, sectionSizeValue) in sectionSize.enumerated() {
            if abs(theOffSet - sectionSizeValue) < errorDelta {
                continue
            }
            if theOffSet < sectionSizeValue {
                val = index
                break
            }
        }
        return val
    }

    func startIndexBinarySearch<T: Comparable>(_ val: [T], offset: T) -> Int {
        if val.count < 3 {
            return 0
        } // If the range is less than 2 just break here.
        var midIndex: Int = 0
        var startIndex = 0
        var endIndex = val.count - 1
        while startIndex < endIndex {
            midIndex = startIndex + (endIndex - startIndex) / 2
            if midIndex + 1  >= val.count || offset >= val[midIndex] &&
                offset < val[midIndex + 1] ||  val[midIndex] == offset {
                    break
            } else if val[midIndex] < offset {
                startIndex = midIndex + 1
            } else {
                endIndex = midIndex
            }
        }
        return midIndex
    }

    /// Returns an object initialized from data in a given unarchiver.
    /// self, initialized using the data in decoder.
    required public init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
    }

    /// Returns the content offset to use after an animation
    /// layout update or change.
    /// - Parameter proposedContentOffset: The proposed point for the
    ///   upper-left corner of the visible content
    /// - returns: The content offset that you want to use instead
    open override func targetContentOffset(forProposedContentOffset proposedContentOffset: CGPoint) -> CGPoint {
        var retval = proposedContentOffset
        if let lastOffsetIndex = delegate.lastIndexOffset {
            delegate.lastIndexOffset = nil

            switch lastOffsetIndex.1 {
            case .supplementaryView:
                if let headerAttr = layoutAttributesForSupplementaryView(ofKind: UICollectionElementKindSectionHeader, at: lastOffsetIndex.0) {
                    retval = scrollDirection == .horizontal ? CGPoint(x: headerAttr.frame.origin.x, y: 0) : CGPoint(x: 0, y: headerAttr.frame.origin.y)
                }
            case .cell:
                if let cellAttr = layoutAttributesForItem(at: lastOffsetIndex.0) {
                    retval = scrollDirection == .horizontal ? CGPoint(x: cellAttr.frame.origin.x, y: 0) : CGPoint(x: 0, y: cellAttr.frame.origin.y)
                }
            default:
                break
            }
            
            // Floating point issues. number could appear the same, but are not.
            // thereby causing UIScollView to think it has scrolled
            let retvalOffset: CGFloat
            let calendarOffset: CGFloat
            
            switch scrollDirection {
                case .horizontal:
                    retvalOffset = retval.x
                    calendarOffset = collectionView!.contentOffset.x
                case .vertical:
                    retvalOffset = retval.y
                    calendarOffset = collectionView!.contentOffset.y
            }
            
            if  abs(retvalOffset - calendarOffset) < errorDelta {
                retval = collectionView!.contentOffset
            }
        }
        return retval
    }

    func clearCache() {
        headerCache.removeAll()
        cellCache.removeAll()
        sectionSize.removeAll()
        currentHeader = nil
        currentCell = nil
        lastWrittenCellAttribute = nil
        xCellOffset = 0
        yCellOffset = 0
        contentHeight = 0
        contentWidth = 0
        stride = 0
    }
}
